// @ myth - client - input.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk
//
// This file contain the input handler.
//
// The input handler is responsible from maintaining and answering the
// requests from the server.
//
// The three main functions of the input handler is:
//    1) Collect request from the server.
//          ask(channel), Read a request from channel.
//          PutPage(string), append a line to the page connected with the
//                           last request.
//          bool ready(), TRUE when a request has arrived
//    2) Collecting answers from the the keyboard.
//       read(key, paging), use key as part of an answer (or to shift
//       		    page, iff paging == TRUE)
//       update(output), update the screen (prompt for answer to request).
//       bool check(output), check for local commands. If the first
//                           entry in the queue is a local command,
//                           this function will execute it, and then
//                           return TRUE.
//       bool active(),  TRUE, iff the request haven't been answered.
//    3) Send answer to the server (or execute local commands).
//	    answer(channel), send the answer to the request to channel.
//
// After the game, EndGame() can be used to allow the player to read
// news messages  etc.

// @@ begin

#ifndef INPUT_HEADER
#define INPUT_HEADER

// @@ include

#include "common/common.h"
#include "common/page.h"

struct OUTPUT;
struct CHANNEL;
struct QUESTION;

// @@ INPUT

class INPUT
{
    PAGE page;			// Information following the request
    bool do_escape;		// Check for special keys
    bool do_redraw;
    bool do_help;
    bool do_page;
    bool do_update;		// TRUE, iff read has been called after update
    const char* error_value;	// Any error happened?
    void clear();		// Remove question
    QUESTION* question;		// Creating an answer to the request
    long argument;		// The argument following the answer
    QUESTION* current();	// The current sub-question to be aswered
public:
    // * Get request
    void ask(CHANNEL* channel);	// Recieve an request from channel
    void PutPage(STRING string)	// More text following the request
    { page.append(string); }
    bool ready()		// Check if any request has been made
    { return question != NULL; }
    // * Read the answer
    void read(char c, bool paging); // Read key from keyboard
    void update(OUTPUT* output); // Update screen
    bool check(OUTPUT* output);	// Handle local commands
    bool active()		// Is the request answered yet?
    { return current() != NULL; }
    // * Answer the request
    void answer(CHANNEL* channel); // Send answer to server
    // * Administradia
    void EndGame();		// Allow player to finish the game
    INPUT();
};

// @@ end

#endif INPUT_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
