// @ myth - client - question.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/page.h"
#include "client/question.h"
#include "common/error.h"
#include "client/macro.h"
#include "common/format.h"
#include "client/global.h"

// @@ QUESTION

// @@@ class QUESTION

// @@@@ const char* QUESTION::read(MACRO*, long&)

const char* QUESTION::read(MACRO*, long&)
{
    bug.error("virtual void QUESTION::read", "Illegal call");
    return "Internal error found, sorry!";
}

// @@@@ void QUESTION::answer(CHANNEL*)

void QUESTION::answer(CHANNEL*)
{ bug.error("QUESTION::answer", "Illegal call"); }

// @@@@ void QUESTION::update(OUTPUT*, long, const char*)

void QUESTION::update(OUTPUT*, long, const char*)
{ bug.error("QUESTION::update", "Illegal call"); }

// @@@@ void QUESTION::help(OUTPUT*)

void QUESTION::help(OUTPUT*)
{ bug.error("QUESTION::help", "Illegal call"); }

// @@@@ bool QUESTION::check(OUTPUT*, long)

bool QUESTION::check(OUTPUT*, long)
{ return FALSE; }

// @@@@ QUESTION::QUESTION(prompt_type prompt)

QUESTION::QUESTION(prompt_type type)
{
    is_done = FALSE;
    prompt = type;
}

// @@@ class QUESTION_OBJECT

// @@@@ QUESTION* QUESTION_OBJECT::current()

QUESTION* QUESTION_OBJECT::current()
{
    if(!symbol->done())
	return symbol;
    else if(!IsKind(symbol->what()))
	return NULL;
    else if(letter == NULL)
    {
	letter = new QUESTION_LETTER(prompt, bunch, symbol->what());
	return letter;
    }
    else if(!letter->done())
	return letter;
    else
	return NULL;
}

// @@@@ void QUESTION_OBJECT::answer(CHANNEL* channel)

void QUESTION_OBJECT::answer(CHANNEL* channel)
{
    if(symbol == NULL)
    {
	bug.error("QUESTION_OBJECT::answer", "NULL symbol");
	channel->PutChar(SYMBOL_ERROR);
    }
    else
    {
	channel->PutChar(symbol->what());

	if(IsKind(symbol->what()))
	{
	    if(letter == NULL)
	    {
		bug.error("QUESTION_OBJECT::answer", "NULL letter");
		channel->PutChar(SYMBOL_ERROR);
	    }
	    else
		channel->PutChar(letter->what());
	}
    }
}

// @@@@ QUESTION_OBJECT::QUESTION_OBJECT(prompt_type prompt,

QUESTION_OBJECT::QUESTION_OBJECT(prompt_type prompt,
				 inventory_type bunch) : (prompt)
{
    is_done = TRUE;		// Move control to symbol and letter.
    QUESTION_OBJECT::prompt = prompt;
    QUESTION_OBJECT::bunch = bunch;
    symbol = new QUESTION_SYMBOL(prompt, bunch);
    letter = NULL;
}

// @@@@ QUESTION_OBJECT::~QUESTION_OBJECT()

QUESTION_OBJECT::~QUESTION_OBJECT()
{
    if(symbol != NULL)		// Dump core without this test!
	delete symbol;
    if(letter != NULL)		// Dump core without this test!
	delete letter;
}

// @@@ class QUESTION_POSITION

// @@@@ const char* QUESTION_POSITION::read(MACRO* macro, long&)

const char* QUESTION_POSITION::read(MACRO* macro, long&)
{
    direction d = macro->dir();
    
    if(d == DIRECTION_NOWHERE)
	is_done = TRUE;
    else if(d == DIRECTION_ERROR)
	return "Try a real direction, like the one you get with <TAB>.";
    else if(dz[d] != 0)
	return "You must select a position on this level!";
    else
	position.move(d);

    return NULL;
}

// @@@@ void QUESTION_POSITION::update(OUTPUT* output,

void QUESTION_POSITION::update(OUTPUT* output,
			       long argument, const char* error)
{
    output->prompt.select(prompt, error, PROMPT_POSITION, argument);
    output->prompt.update();
    output->dungeon.flush(position);
}

// @@@ class QUESTION_STRING

// @@@@ const char* QUESTION_STRING::read(MACRO* macro, long&)

const char* QUESTION_STRING::read(MACRO* macro, long&)
{
    if(macro->check(KEY_ACCEPT))
	is_done = TRUE;
    else
	macro->edit(string);

    return NULL;
}

// @@@@ void QUESTION_STRING::update(OUTPUT* output,

void QUESTION_STRING::update(OUTPUT* output, long argument, const char* error)
{
    output->dungeon.update();
    output->prompt.select(prompt, error, PROMPT_STRING, argument, string);
    output->prompt.flush();
}

// @@@ class QUESTION_DIRECTION

// @@@@ const char* QUESTION_DIRECTION::read(MACRO* macro, long&)

const char* QUESTION_DIRECTION::read(MACRO* macro, long&)
{
    dir = macro->dir();

    if(dir == DIRECTION_ERROR)
	return "That's not a valid direction, press <TAB> for a list.";

    is_done = TRUE;
    return NULL;
}

// @@@@ void QUESTION_DIRECTION::update(OUTPUT* output,

void QUESTION_DIRECTION::update(OUTPUT* output,
				long argument, const char* error)
{
    output->dungeon.update();
    output->prompt.select(prompt, error, PROMPT_DIRECTION, argument);
    output->prompt.flush();
}

// @@@ class QUESTION_SYMBOL

// @@@@ const char* QUESTION_SYMBOL::read(MACRO* macro, long&)

const char* QUESTION_SYMBOL::read(MACRO* macro, long&)
{
    symbol = macro->symbol();

    if(symbol == SYMBOL_ERROR)
	return "No objects look like that!";
    
    is_done = TRUE;
    return NULL;
}

// @@@@ void QUESTION_SYMBOL::update(OUTPUT* output,

void QUESTION_SYMBOL::update(OUTPUT* output, long argument, const char* error)
{
    output->dungeon.update();
    output->prompt.select(prompt, error, PROMPT_SYMBOL, argument,
			  output->inventory.symbol(bunch));
    output->prompt.flush();
}

// @@@ class QUESTION_CONFIRMATION

// @@@@ const char* QUESTION_CONFIRMATION::read(MACRO* macro, long&)

const char* QUESTION_CONFIRMATION::read(MACRO* macro, long&)
{
    conf = macro->conf();
    if(conf == CONFIRMATION_ERROR)
	return "Just press 'Y'es, 'N'o or 'Q'uit to answer.";

    is_done = TRUE;
    return NULL;
}

// @@@@ void QUESTION_CONFIRMATION::answer(CHANNEL* channel)

void QUESTION_CONFIRMATION::answer(CHANNEL* channel)
{ channel->PutChar(conf); }

// @@@@ void QUESTION_CONFIRMATION::update(OUTPUT* output,

void QUESTION_CONFIRMATION::update(OUTPUT* output,
				   long argument, const char* error)
{
    output->dungeon.update();
    output->prompt.select(prompt, error, PROMPT_CONFIRMATION, argument);
    output->prompt.flush();
}

// @@@ class QUESTION_ARGUMENT

// @@@@ const char* QUESTION_ARGUMENT::read(MACRO* macro, long& argument)

const char* QUESTION_ARGUMENT::read(MACRO* macro, long& argument)
{
    if(macro->check(KEY_ACCEPT))
	is_done = TRUE;
    else if(macro->check(KEY_ERASE))
	argument /= 10;
    else if(macro->check(KEY_KILL))
	argument = 0;
    else
    {
	(void) macro->key();
	return "That's not a digit.";
    }
    return NULL;
}

// @@@@ void QUESTION_ARGUMENT::update(OUTPUT* output,

void QUESTION_ARGUMENT::update(OUTPUT* output, 
			       long argument, const char* error)
{
    output->dungeon.update();
    output->prompt.select(prompt, error, PROMPT_ARGUMENT, argument);
    output->prompt.flush();
}

// @@@ class QUESTION_LETTER

// @@@@ const char* QUESTION_LETTER::read(MACRO* macro, long&)

const char* QUESTION_LETTER::read(MACRO* macro, long&)
{
    letter = macro->key();
    is_done = TRUE;
    return NULL;
}

// @@@@ void QUESTION_LETTER::update(OUTPUT* output,

void QUESTION_LETTER::update(OUTPUT* output, long argument, const char* error)
{
    output->dungeon.update();
    output->prompt.select(prompt, error, PROMPT_LETTER, argument,
			  output->inventory.letter(bunch, symbol));
    output->prompt.flush();
}

// @@@@ void QUESTION_LETTER::help(OUTPUT* output)

void QUESTION_LETTER::help(OUTPUT* output)
{
    output->inventory.show(bunch, symbol, 0L);
    // Check if a new cover is created.
    if(output->inventory.check())
	output->cover(output->inventory.cover());
}    

// @@@@ QUESTION_LETTER::QUESTION_LETTER(prompt_type prompt,

QUESTION_LETTER::QUESTION_LETTER(prompt_type prompt,
			   inventory_type bunch,
			   symbol_type symbol) : (prompt)
{
    QUESTION_LETTER::bunch = bunch;
    QUESTION_LETTER::symbol = symbol;
    QUESTION_LETTER::letter = EOF;
}

// @@@ class QUESTION_NAME

// @@@@ const char* QUESTION_NAME::read(MACRO* macro, long&)

const char* QUESTION_NAME::read(MACRO* macro, long&)
{
    if(macro->check(KEY_ACCEPT))
	is_done = TRUE;
    else
	macro->edit(string);

    return NULL;
}

// @@@@ void QUESTION_NAME::update(OUTPUT* output,

void QUESTION_NAME::update(OUTPUT* output, long argument, const char* error)
{
    output->dungeon.update();
    output->prompt.select(prompt, error, PROMPT_NAME, argument, string);
    output->prompt.flush();
}

// @@@ class QUESTION_EXTENDED

// @@@@ bool QUESTION_EXTENDED::check(OUTPUT* output, long)

bool QUESTION_EXTENDED::check(OUTPUT* output, long)
{
    if(strcmp(string, "version") == EQUAL)
	output->news.insert(format("client: %s.", MythVersion));
    else if(strncmp(string, "option", 6) == EQUAL)
    {
	if(global.parse(string + 6) == ERROR)
	    // Let us be *really* helpfull here.
	    output->news.insert("syntax error");

	return TRUE;
    }
    return FALSE;
}

// @@@ class QUESTION_NAME


// @@@@ QUESTION_NAME::QUESTION_NAME(prompt_type prompt)

QUESTION_NAME::QUESTION_NAME(prompt_type prompt) : (prompt)
{
    STRING name = global.name();
    (void) strcpy(string, name());
}


// @@@ class QUESTION_COMMAND

// @@@@ QUESTION* QUESTION_COMMAND::current()

QUESTION* QUESTION_COMMAND::current()
{
    if(command == NULL)
    {
	bug.error("QUESTION_COMMAND::current", "NULL command");
	return NULL;
    }
    else if(command->done())
	return NULL;
    else
	return command->current();
}

// @@@@ const char* QUESTION_COMMAND::read(MACRO* macro, long&)

const char* QUESTION_COMMAND::read(MACRO* macro, long&)
{
    switch(macro->command())
    {
    case CLIENT_MOVE:
	command = new COMMAND_MOVE;
	break;
    case CLIENT_RUN:
	command = new COMMAND_RUN;
	break;
    case CLIENT_DROP:
	command = new COMMAND_DROP;
	break;
    case CLIENT_GET:
	command = new COMMAND_GET;
	break;
    case CLIENT_APPLY:
	command = new COMMAND_APPLY;
	break;
    case CLIENT_THROW:
	command = new COMMAND_THROW;
	break;
    case CLIENT_SLEEP:
	command = new COMMAND_SLEEP;
	break;
    case CLIENT_COMMAND:
	command = new COMMAND_COMMAND;
	break;
    case CLIENT_RECENTER:
	command = new COMMAND_RECENTER;
	break;
    case CLIENT_INVENTORY:
	command = new COMMAND_INVENTORY;
	break;
    case CLIENT_SHOW_STATUS:
	command = new COMMAND_STATUS;
	break;
    case CLIENT_READ_NEWS:
	command = new COMMAND_NEWS;
	break;
    case CLIENT_NEWS_UP:
	command = new COMMAND_UP;
	break;
    case CLIENT_NEWS_DOWN:
	command = new COMMAND_DOWN;
	break;
    case CLIENT_IDLE:
    case CLIENT_END:
    case CLIENT_ERROR:
    default:
	return "That key is not bound to anything, press <TAB> for a list.";
    }
    is_done = TRUE;
    return NULL;
}

// @@@@ void QUESTION_COMMAND::answer(CHANNEL* channel)

void QUESTION_COMMAND::answer(CHANNEL* channel)
{
    if(command == NULL)
	bug.error("QUESTION_COMMAND::answer", "NULL command");
    else
	command->answer(channel);
}

// @@@@ void QUESTION_COMMAND::update(OUTPUT* output,

void QUESTION_COMMAND::update(OUTPUT* output, long argument, const char* error)
{
    output->prompt.select(prompt, error, PROMPT_COMMAND, argument);
    output->prompt.update();
    output->dungeon.flush(Origo);
}

// @@@@ bool QUESTION_COMMAND::check(OUTPUT* output, long argument)

bool QUESTION_COMMAND::check(OUTPUT* output, long argument)
{
    if(command == NULL)
	return FALSE;
    else if(command->check(output, argument))
    {
	delete command;
	command = NULL;
	argument = 0L;
	is_done = FALSE;
	return TRUE;
    }
    else
	return FALSE;
}

// @@@@ QUESTION_COMMAND::~QUESTION_COMMAND()

QUESTION_COMMAND::~QUESTION_COMMAND()
{
    if(command != NULL)		// Shouldn't be needed, but is!
	delete command;
}

// @@@ class QUESTION_GAME_OVER

// @@@@ const char* QUESTION_GAME_OVER::read(MACRO* macro, long&)

const char* QUESTION_GAME_OVER::read(MACRO* macro, long&)
{
    switch(macro->command())
    {
    case CLIENT_MOVE:
    case CLIENT_RUN:
    case CLIENT_DROP:
    case CLIENT_GET:
    case CLIENT_APPLY:
    case CLIENT_THROW:
    case CLIENT_COMMAND:
	return "You are not playing any more!";
    case CLIENT_SLEEP:
	/* Stop the program for good! */
	command = new COMMAND_SLEEP;
	break;
    case CLIENT_RECENTER:
	command = new COMMAND_RECENTER;
	break;
    case CLIENT_INVENTORY:
	command = new COMMAND_INVENTORY;
	break;
    case CLIENT_SHOW_STATUS:
	command = new COMMAND_STATUS;
	break;
    case CLIENT_READ_NEWS:
	command = new COMMAND_NEWS;
	break;
    case CLIENT_NEWS_UP:
	command = new COMMAND_UP;
	break;
    case CLIENT_NEWS_DOWN:
	command = new COMMAND_DOWN;
	break;
    case CLIENT_IDLE:
    case CLIENT_END:
    case CLIENT_ERROR:
    default:
	return "That key is not bound to anything, press <TAB> for a list.";
    }
    is_done = TRUE;
    return NULL;
}

// @@ COMMAND

// @@@ class COMMAND

// @@@@ bool COMMAND::check(OUTPUT*, long)
//
// Default, for commands adressed to the server.

bool COMMAND::check(OUTPUT*, long)
{ return FALSE; }

// @@@ class COMMAND_EXTERN

// @@@@ QUESTION* COMMAND_EXTERN::current()

QUESTION* COMMAND_EXTERN::current()
{
    for(int i = 0; i < MaxCommandLength && arg[i] != NULL; i++)
    {
	if(arg[i]->done())
	{
	    if(arg[i]->current() == NULL)
		/* Do Nothing */;
	    else
		return arg[i]->current();
	}
	else
	    return arg[i];
    }
    return NULL;
}

// @@@@ bool COMMAND_EXTERN::check(OUTPUT* output, long argument)

bool COMMAND_EXTERN::check(OUTPUT* output, long argument)
{
    bool IsLocal = TRUE;
    
    for(int i = 0; i < MaxCommandLength && arg[i] != NULL; i++)
    {
	if(arg[i]->check(output, argument) == FALSE)
	    IsLocal = FALSE;
    }
    return IsLocal;
}

// @@@@ void COMMAND_EXTERN::answer(CHANNEL* channel)

void COMMAND_EXTERN::answer(CHANNEL* channel)
{
    channel->PutChar(type());
    
    for(int i = 0; i < MaxCommandLength && arg[i] != NULL; i++)
	arg[i]->answer(channel);
}

// @@@@ COMMAND_EXTERN::COMMAND_EXTERN()

COMMAND_EXTERN::COMMAND_EXTERN()
{
    for(int i = 0; i < MaxCommandLength; i++)
	arg[i] = NULL;
}

// @@@@ COMMAND_EXTERN::~COMMAND_EXTERN()

COMMAND_EXTERN::~COMMAND_EXTERN()
{
    for(int i = 0; i < MaxCommandLength && arg[i] != NULL; i++)
	delete arg[i];
}

// @@@ class COMMAND_LOCAL

// @@@@ void COMMAND_LOCAL::execute(OUTPUT*, long)

void COMMAND_LOCAL::execute(OUTPUT*, long)
{ bug.error("COMMAND_LOCAL::execute", "virtual"); }

// @@@ class COMMAND_INVENTORY()

COMMAND_INVENTORY::COMMAND_INVENTORY()
{ symbol = new QUESTION_SYMBOL(PROMPT_INVENTORY, INVENTORY_OWN); }

COMMAND_INVENTORY::~COMMAND_INVENTORY()
{ delete symbol; }

// @@@ class COMMAND_RECENTER()

// @@@ class COMMAND_RECENTER

// @@@@ void COMMAND_RECENTER::execute(OUTPUT* output, long argument)

void COMMAND_RECENTER::execute(OUTPUT* output, long)
{ output->dungeon.center(position->where()); }

// @@@@ COMMAND_RECENTER::COMMAND_RECENTER()

COMMAND_RECENTER::COMMAND_RECENTER()
{ position = new QUESTION_POSITION(PROMPT_RECENTER); }

// @@@@ COMMAND_RECENTER::~COMMAND_RECENTER()

COMMAND_RECENTER::~COMMAND_RECENTER()
{ delete position; }

// @@@ class COMMAND_NEWS

// @@@@ void COMMAND_NEWS::execute(OUTPUT* output, long)

void COMMAND_NEWS::execute(OUTPUT* output, long)
{ output->news.read(); }

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
