// @ myth - server - beast.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk
//
// The beast (as an object in the game) is defined in this file.

// @@ begin

#ifndef BEAST_HEADER
#define BEAST_HEADER

// @@ include

#include "server/monster.h"

// @@ enum move_tactic

enum move_tactic
{
    MOVE_FOLLOW,
    MOVE_NOWHERE,
    MOVE_INDEPENDENT,
    MOVE_ENEMY,
    MOVE_END,
    MOVE_ERROR = ERROR
};

// @@ BEAST

class BEAST : public MONSTER
{
private:
    coor target;		// Where to move
    long MinWorth;		// Wont pick up anything worth less
    cont UseItem(LEVEL* level, symbol_type symbol); // Use best item from pack
protected:
    ID_LIST enemies;		// List of known enemies of this monster
    virtual void SpecialAction(LEVEL*) // Special actions by special monsters
    { /* Default: none */ }
    virtual move_tactic tactic() // What tactic does the monster use to move?
    { return MOVE_FOLLOW; }
    virtual bool UseMagic()	// Does this monster use magic?
    { return FALSE; }
    virtual bool UseWeapon()	// Does this monster use weapons?
    { return FALSE; }
    virtual bool UseFood()	// Does it eat?
    { return TRUE; }
    virtual bool UseThing();	// Any interest in objects at all?
public:
    bool IsEnemy(OBJECT* object);
    bool IsPicking(OBJECT* object);
    MONSTER* enemy(LEVEL* level);
    STRING hurt(int power, OBJECT* object, MONSTER* monster, LEVEL* level);
    cont play(LEVEL* level);	// Move the beast
    void save(BINARY* binary);
    BEAST(BINARY* binary);
    BEAST(PARSE* parse, coor pos);
    BEAST();
    ~BEAST();
};

// @@ WIZARD

class WIZARD : public BEAST
{
private:
    ID_LIST talker;
    ID_LIST amulet;
    void create();
protected:
    void SpecialAction(LEVEL* level);
    move_tactic tactic()
    { return MOVE_INDEPENDENT; }
    bool UseMagic()
    { return TRUE; }
    bool UseWeapon()
    { return TRUE; }
public:
    bool IsUnique()
    { return TRUE; }
    const char* name();
    bool IsEnemy(OBJECT* object);
    bool IsPicking(OBJECT*)
    { return body->wounded(); }
    object_type type()		// This is a beast
    { return MONSTER_WIZARD; }
    int knowledge(OBJECT*)
    { return 100; }
    bool know(OBJECT*)
    { return TRUE; }
    int dexterity()
    { return 25; }
    void save(BINARY* binary);
    WIZARD(BINARY* binary);
    WIZARD(PARSE* parse, coor pos) : (parse, pos) { create(); }
    WIZARD() { create(); }
};

// @@ ACID_BLOB

class ACID_BLOB : public BEAST
{
    void create();
protected:
    bool UseFood()
    { return FALSE; }
public:
    object_type type()		// This is a beast
    { return MONSTER_ACID_BLOB; }
    int dexterity()
    { return 1; }
    ACID_BLOB(BINARY* binary) : (binary) { }
    ACID_BLOB(PARSE* parse, coor pos) : (parse, pos) { create(); }
    ACID_BLOB() { create(); }
};

// @@ BAT

class BAT : public BEAST
{
    void create();
public:
    object_type type()		// This is a beast
    { return MONSTER_BAT; }
    int dexterity()
    { return 3; }
    BAT(BINARY* binary) : (binary) { }
    BAT(PARSE* parse, coor pos) : (parse, pos) { create(); }
    BAT() { create(); }
};

// @@ RAT

class RAT : public BEAST
{
    void create(int exp);
public:
    bool IsWanted(OBJECT* object);
    object_type type()		// This is a beast
    { return MONSTER_RAT; }
    int knowledge(OBJECT*object);
    bool know(OBJECT* object);
    int dexterity()
    { return 10; }
    RAT(BINARY* binary) : (binary) { }
    RAT(PARSE* parse, coor pos) : (parse, pos)
    { create(2); }
    RAT(int exp = 2)
    { create(exp); }
};

// @@ GOBLIN

class GOBLIN : public BEAST
{
    void create();
protected:
    bool UseWeapon()
    { return TRUE; }
public:
    object_type type()		// This is a beast
    { return MONSTER_GOBLIN; }
    int knowledge(OBJECT*object);
    int dexterity()
    { return 7; }
    GOBLIN(BINARY* binary) : (binary) { }
    GOBLIN(PARSE* parse, coor pos) : (parse, pos) { create(); }
    GOBLIN() { create(); }
};

// @@ HOBGOBLIN

class HOBGOBLIN : public BEAST
{
    void create();
protected:
    bool UseMagic()
    { return TRUE; }
    bool UseWeapon()
    { return TRUE; }
public:
    object_type type()
    { return MONSTER_HOBGOBLIN; }
    int knowledge(OBJECT*object);
    int dexterity()
    { return 12; }
    HOBGOBLIN(BINARY* binary) : (binary) { }
    HOBGOBLIN(PARSE* parse, coor pos) : (parse, pos) { create(); }
    HOBGOBLIN() { create(); }
};

// @@ ORC

class ORC : public BEAST
{
    void create();
protected:
    bool UseWeapon()
    { return TRUE; }
public:
    bool IsEnemy(OBJECT* object);
    object_type type()		// This is a beast
    { return MONSTER_ORC; }
    int knowledge(OBJECT*object);
    int dexterity()
    { return 10; }
    ORC(BINARY* binary) : (binary) { }
    ORC(PARSE* parse, coor pos) : (parse, pos) { create(); }
    ORC() { create(); }
};

// @@ DRAGON

class DRAGON : public BEAST
{
    void create();
protected:
    void SpecialAction(LEVEL* level);
    move_tactic tactic()
    { return MOVE_INDEPENDENT; }
    bool UseMagic()
    { return TRUE; }
public:
    object_type type()		// This is a beast
    { return MONSTER_DRAGON; }
    int knowledge(OBJECT*object);
    bool know(OBJECT* object);
    int dexterity()
    { return 20; }
    DRAGON(BINARY* binary) : (binary) { }
    DRAGON(PARSE* parse, coor pos) : (parse, pos) { create(); }
    DRAGON() { create(); }
};

// @@ LIZARD

class LIZARD : public BEAST
{
    void create();
public:
    object_type type()		// This is a beast
    { return MONSTER_LIZARD; }
    int dexterity()
    { return 5; }
    LIZARD(BINARY* binary) : (binary) { }
    LIZARD(PARSE* parse, coor pos) : (parse, pos) { create(); }
    LIZARD() { create(); }
};

// @@ FUNGUS

class FUNGUS : public BEAST
{
    void create();
protected:
    bool UseFood()
    { return FALSE; }
    move_tactic tactic()
    { return MOVE_NOWHERE; }
public:
    bool IsEnemy(OBJECT* object);
    object_type type()		// This is a beast
    { return MONSTER_FUNGUS; }
    int dexterity()
    { return 10; }
    FUNGUS(BINARY* binary) : (binary) { }
    FUNGUS(PARSE* parse, coor pos) : (parse, pos) { create(); }
    FUNGUS() { create(); }
};

// @@ TROLL

class TROLL : public BEAST
{
    void create();
protected:
    bool UseWeapon()
    { return TRUE; }
public:
    object_type type()		// This is a beast
    { return MONSTER_TROLL; }
    int knowledge(OBJECT*object);
    int dexterity()
    { return 5; }
    TROLL(BINARY* binary) : (binary) { }
    TROLL(PARSE* parse, coor pos) : (parse, pos) { create(); }
    TROLL() { create(); }
};

// @@ COCKATRICE

class COCKATRICE : public BEAST
{
    void create();
protected:
    bool UseFood()
    { return FALSE; }
    void SpecialAction(LEVEL* level);
    move_tactic tactic()
    { return MOVE_INDEPENDENT; }
public:
    object_type type()		// This is a beast
    { return MONSTER_COCKATRICE; }
    int dexterity()
    { return 6; }
    COCKATRICE(BINARY* binary) : (binary) { }
    COCKATRICE(PARSE* parse, coor pos) : (parse, pos) { create(); }
    COCKATRICE() { create(); }
};

// @@ WRAITH

class WRAITH : public BEAST
{
    void create();
protected:
    bool UseFood()
    { return FALSE; }
    bool UseMagic()
    { return TRUE; }
public:
    bool IsWanted(OBJECT* object);
    object_type type()		// This is a beast
    { return MONSTER_WRAITH; }
    int knowledge(OBJECT*object);
    bool know(OBJECT* object);
    int dexterity()
    { return 17; }
    WRAITH(BINARY* binary) : (binary) { }
    WRAITH(PARSE* parse, coor pos) : (parse, pos) { create(); }
    WRAITH() { create(); }
};

// @@ ELF

class ELF : public BEAST
{
    void create();
protected:
    move_tactic tactic()
    { return MOVE_ENEMY; }
    bool UseMagic()
    { return TRUE; }
    bool UseWeapon()
    { return TRUE; }
public:
    bool IsEnemy(OBJECT* object);
    object_type type()		// This is a beast
    { return MONSTER_ELF; }
    int knowledge(OBJECT*object);
    bool know(OBJECT* object);
    int dexterity()
    { return 22; }
    ELF(BINARY* binary) : (binary) { }
    ELF(PARSE* parse, coor pos) : (parse, pos) { create(); }
    ELF() { create(); }
};

// @@ end

#endif BEAST_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
