// @ myth - server - ring.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/ring.h"
#include "server/monster.h"
#include "server/ability.h"
#include "common/error.h"
#include "server/rnd.h"

// @@ void RING_HEALTH::InsertAbility(MONSTER* monster)

void RING_HEALTH::InsertAbility(MONSTER* monster)
{
    if(!monster->pack.worn(this))
	return;
    
    switch(holy)
    {
    case CURSED:
	monster->pack.AddAbility(new COUNTER_HUNGER(0L, MyValue));
	break;
    case NORMAL:
	monster->pack.AddAbility(new COUNTER_HUNGER(0L, MyValue));
	monster->pack.AddAbility(new COUNTER_HEALTH(0L, MyValue));
	// Fall through
    case BLESSED:
	monster->pack.AddAbility(new COUNTER_HEALTH(0L, MyValue));
	break;
    default:
	bug.error("RING_HEALTH::InsertAbility", "Blasphemy");
    }
}

// @@ void RING_TELEPORTATION::InsertAbility(MONSTER* monster)

void RING_TELEPORTATION::InsertAbility(MONSTER* monster)
{
    if(!monster->pack.worn(this))
	return;
    
    switch(holy)
    {
    case BLESSED:
	monster->pack.AddAbility(new COUNTER_TELEPORT_CONTROL);
	break;
    case NORMAL:
    case CURSED:
	monster->pack.AddAbility(new COUNTER_RANDOM_TELEPORT(0L, MyValue));
	break;
    default:
	bug.error("RING_TELEPORTATION::InsertAbility", "Blasphemy");
    }

}

// @@ void RING_VISION::InsertAbility(MONSTER* monster)

void RING_VISION::InsertAbility(MONSTER* monster)
{
    if(!monster->pack.worn(this))
	return;
    
    switch(holy)
    {
    case CURSED:
	monster->pack.AddAbility(new COUNTER_BLIND);
	break;
    case NORMAL:
	monster->pack.AddAbility(new COUNTER_SEE_INVISIBLE);
	// Fall through
    case BLESSED:
	monster->pack.AddAbility(new COUNTER_INVISIBLE);
	break;
    default:
	bug.error("RING_VISION::InsertAbility", "Blasphemy");
    }
}

// @@ void RING_STRENGTH::InsertAbility(MONSTER* monster)

void RING_STRENGTH::InsertAbility(MONSTER* monster)
{
    if(monster->pack.worn(this))
	monster->pack.AddAbility(new COUNTER_STRENGTH(0L, MyValue / 2 + 1));
}

// @@ cont RING_FIRE_RESISTANCE::absorb(int* power,

cont RING_FIRE_RESISTANCE::absorb(int* power, damage_type* type,
				  OBJECT*, MONSTER*, MONSTER* monster)
{
    if(!monster->pack.worn(this))
	/* Do Nothing */;
    else if(rnd.chance(MyValue, MaxValue))
	*type &= ~DAMAGE_FLAME;
    else if(*type == DAMAGE_FLAME)
    {
	*power *= MaxValue - rnd(0, MyValue);
	*power /= MaxValue;
    }
    return CONTINUE;
}

// @@ cont RING_COLD_RESISTANCE::absorb(int* power,

cont RING_COLD_RESISTANCE::absorb(int* power, damage_type* type,
				  OBJECT*, MONSTER*, MONSTER* monster)
{
    if(!monster->pack.worn(this))
	/* Do Nothing */;
    else if(rnd.chance(MyValue, MaxValue))
	*type &= ~DAMAGE_FLAME;
    else if(*type == DAMAGE_FLAME)
    {
	*power *= MaxValue - rnd(0, MyValue);
	*power /= MaxValue;
    }
    return CONTINUE;
}

// @@ void AMULET_YENDOR::InsertAbility(MONSTER* monster)

void AMULET_YENDOR::InsertAbility(MONSTER* monster)
{
    monster->pack.AddAbility(new COUNTER_YENDOR);
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
