// @ myth - server - wand.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef WAND_HEADER
#define WAND_HEADER

// @@ include

#include "server/thing.h"
#include "server/ray.h"

// @@ WAND_SLEEP

class WAND_SLEEP : public WAND
{
protected:
    RAY* RayCursed()
    { return new RAY_FAST(MyValue); }
    RAY* RayNormal()
    { return new RAY_SLOW(MyValue); }
    RAY* RayBlessed()
    { return new RAY_SLEEP(MyValue); }
public:
    // Identification
    object_type type()
    { return WAND_OF_SLEEP; }
    const char* MagicName()
    { return "sleep"; }
    // Administration
    OBJECT* split(long arg)
    { MyCount -= arg; return new WAND_SLEEP(this, arg); }
    WAND_SLEEP(BINARY* binary) : (binary) { }
    WAND_SLEEP(PARSE* parse) : (parse) { }
    WAND_SLEEP(WAND_SLEEP* thing, long count) : (thing, count) { }
    WAND_SLEEP(long count = 1L) : (count) { }
};

// @@ WAND_LIFE

class WAND_LIFE : public WAND
{
protected:
    RAY* RayCursed()
    { return new RAY_DEATH(MyValue); }
    RAY* RayNormal()
    { return new RAY_LIFE(MyValue); }
    RAY* RayBlessed()
    { return new RAY_POLYMORPH(MyValue); }
public:
    // Identification
    object_type type()
    { return WAND_OF_LIFE; }
    const char* MagicName()
    { return "life"; }
    // Administration
    OBJECT* split(long arg)
    { MyCount -= arg; return new WAND_LIFE(this, arg); }
    WAND_LIFE(BINARY* binary) : (binary) { }
    WAND_LIFE(PARSE* parse) : (parse) { }
    WAND_LIFE(WAND_LIFE* thing, long count) : (thing, count) { }
    WAND_LIFE(long count = 1L) : (count) { }
};

// @@ WAND_TELEPORT

class WAND_TELEPORT : public WAND
{
protected:
    RAY* RayCursed()
    { return new RAY_APPORT(MyValue); }
    RAY* RayNormal()
    { return new RAY_TELEPORT(MyValue); }
    RAY* RayBlessed()
    { return new RAY_LEVEL_TELEPORT(MyValue); }
public:
    // Identification
    object_type type()
    { return WAND_OF_TELEPORT; }
    const char* MagicName()
    { return "teleport"; }
    // Administration
    OBJECT* split(long arg)
    { MyCount -= arg; return new WAND_TELEPORT(this, arg); }
    WAND_TELEPORT(BINARY* binary) : (binary) { }
    WAND_TELEPORT(PARSE* parse) : (parse) { }
    WAND_TELEPORT(WAND_TELEPORT* thing, long count) : (thing, count) { }
    WAND_TELEPORT(long count = 1L) : (count) { }
};

// @@ WAND_VISIBILITY

class WAND_VISIBILITY : public WAND
{
protected:
    RAY* RayCursed()
    { return new RAY_INVISIBLE(MyValue); }
    RAY* RayNormal()
    { return new RAY_VISIBLE(MyValue); }
    RAY* RayBlessed()
    { return new RAY_LIT(MyValue); }
public:
    // Identification
    object_type type()
    { return WAND_OF_VISIBILITY; }
    const char* MagicName()
    { return "visibility"; }
    // Administration
    OBJECT* split(long arg)
    { MyCount -= arg; return new WAND_VISIBILITY(this, arg); }
    WAND_VISIBILITY(BINARY* binary) : (binary) { }
    WAND_VISIBILITY(PARSE* parse) : (parse) { }
    WAND_VISIBILITY(WAND_VISIBILITY* thing, long count) : (thing, count) { }
    WAND_VISIBILITY(long count = 1L) : (count) { }
};

// @@ WAND_MISSILE

class WAND_MISSILE : public WAND
{
protected:
    RAY* RayCursed()
    { return new RAY_COLD(MyValue); }
    RAY* RayNormal()
    { return new RAY_MAGIC_MISSILE(MyValue); }
    RAY* RayBlessed()
    { return new RAY_FLAME(MyValue); }
public:
    // Identification
    object_type type()
    { return WAND_OF_MISSILE; }
    const char* MagicName()
    { return "missile"; }
    // Administration
    OBJECT* split(long arg)
    { MyCount -= arg; return new WAND_MISSILE(this, arg); }
    WAND_MISSILE(BINARY* binary) : (binary) { }
    WAND_MISSILE(PARSE* parse) : (parse) { }
    WAND_MISSILE(WAND_MISSILE* thing, long count) : (thing, count) { }
    WAND_MISSILE(long count = 1L) : (count) { }
};

// @@ WAND_LIGHT

class WAND_LIGHT : public WAND
{
protected:
    RAY* RayCursed()
    { return new RAY_DARK(MyValue); }
    RAY* RayNormal()
    { return new RAY_LIGHTENING_BOLT(MyValue); }
    RAY* RayBlessed()
    { return new RAY_LIGHT(MyValue); }
public:
    // Identification
    object_type type()
    { return WAND_OF_LIGHT; }
    const char* MagicName()
    { return "light"; }
    // Administration
    OBJECT* split(long arg)
    { MyCount -= arg; return new WAND_LIGHT(this, arg); }
    WAND_LIGHT(BINARY* binary) : (binary) { }
    WAND_LIGHT(PARSE* parse) : (parse) { }
    WAND_LIGHT(WAND_LIGHT* thing, long count) : (thing, count) { }
    WAND_LIGHT(long count = 1L) : (count) { }
};

// @@ end

#endif WAND_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
